@@ -1082,7 +1082,7 @@ class ServiceStub(ABC):
         
         logger.info(f'[响应结果]:{r_data}')
         logger.info(f'[响应http_code]:{r.status_code}')
-        logger.info(f'[响应耗时]:{r.elapsed.total_seconds()}秒')
+        logger.info(f'[响应耗时]:{r.elapsed.total_seconds():0.2f}秒')
         logger.info(f'[X-Trace-Id]:{r.headers.get("X-Trace-Id")}')
         # logger.info(f'[响应cookies]:{r.cookies}')
         response.__dict__.update(r.__dict__)
